@@ -7,9 +7,12 @@
 #ifndef __ASM_ARM_DMA_MAPPING_H
 #define __ASM_ARM_DMA_MAPPING_H
 
+#include <common.h>
+#include <asm/cache.h>
+#include <cpu_func.h>
 #include <linux/dma-direction.h>
-
-#define	dma_mapping_error(x, y)	0
+#include <linux/types.h>
+#include <malloc.h>
 
 static inline void *dma_alloc_coherent(size_t len, unsigned long *handle)
 {
@@ -22,15 +25,4 @@ static inline void dma_free_coherent(void *addr)
 	free(addr);
 }
 
-static inline unsigned long dma_map_single(volatile void *vaddr, size_t len,
-					   enum dma_data_direction dir)
-{
-	return (unsigned long)vaddr;
-}
-
-static inline void dma_unmap_single(volatile void *vaddr, size_t len,
-				    unsigned long paddr)
-{
-}
-
 #endif /* __ASM_ARM_DMA_MAPPING_H */
